<?php


namespace Fastrack\Property;


use Fastrack\Utilities\ModelUtilities;
use Fastrack\Utilities\Slug;
use PlatoCreative\Imgix\Imgix;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\ORM\DataObject;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Security\Security;

class Review extends DataObject implements PermissionProvider
{

	private static $table_name = 'reviews';

	private static $db = [
		'Title' => 'Varchar(255)',
		'Slug' => 'Varchar(255)',
		'Review' => 'Text',
		'Date' => 'Date',
		'Author' => 'Varchar(255)',
	];

	private static $has_one = [
		'Property' => Property::class,
		'Image' => Imgix::class,
	];

	private static $owns = [
		'Image',
	];

	private static $indexes = [
		'Slug' => true,
	];

	private static $translate = [
		'Title',
		'Review',
		'Date',
		'Author',
	];

	public function getCMSFields()
	{
		$fields = parent::getCMSFields();
		ModelUtilities::hideHasOneFields($this, $fields);
		return $fields;
	}

	public function onBeforeWrite()
	{
		$this->Slug = Slug::generateSlug($this->Title, $this->Slug);
		parent::onBeforeWrite();
	}

	public function CMSEditLink()
	{
		return Controller::join_links([
			$this->Property()->CMSEditLink(),
			'ItemEditForm',
			'field',
			'Reviews',
			'item',
			$this->ID,
		]);
	}

	public function WorkflowLink()
	{
		return Controller::join_links([Director::absoluteBaseURL(), $this->CMSEditLink()]);
	}

	public function providePermissions()
	{
		$permisions = [
			'MANAGE_REVIEWS' => [
				'name' => 'Manage reviews',
				'category' => 'Properties',
			],
		];
		$this->extend('updatePermisions', $permisions);
		return $permisions;
	}

	public function canView($member = null)
	{
		return Permission::check(['MANAGE_REVIEWS']);
	}

	public function canEdit($member = null)
	{
		return Permission::check(['MANAGE_REVIEWS']);
	}

	public function canDelete($member = null)
	{
		return Permission::check(['MANAGE_REVIEWS']);
	}

	public function canCreate($member = null, $context = array())
	{
		return Permission::check(['MANAGE_REVIEWS']);
	}
}
